@@ -46,6 +46,7 @@ select softwaresource.branch_url as git_url,
        softwaresource.branch_revision as git_commit_id,
        coalesce(namedattribute_git_describe.value, softwaresource.branch_revision) as git_describe,
        coalesce(namedattribute_git_log_info.value, softwaresource.branch_revision) as git_log_info,
+       namedattribute_gitweb_url.value as gitweb_url,
        softwaresource.commit_timestamp as commit_timestamp,
        testrun.analyzer_assigned_date as build_date,
        namedattribute_kernelconfig.value as config,
@@ -83,6 +84,13 @@ left outer join dashboard_app_namedattribute as namedattribute_git_log_info
             and namedattribute_git_log_info.content_type_id = (
                 select django_content_type.id from django_content_type
                 where app_label = 'dashboard_app' and model='testrun')
+            )
+left outer join dashboard_app_namedattribute as namedattribute_gitweb_url
+        on (namedattribute_gitweb_url.object_id = testrun.id
+            and namedattribute_gitweb_url.name = 'kernel.gitweb_url'
+            and namedattribute_gitweb_url.content_type_id = (
+                select django_content_type.id from django_content_type
+                where app_label = 'dashboard_app' and model='testrun')
             ),
        dashboard_app_softwaresource as softwaresource,
        dashboard_app_testrun_sources as tr_ss_link
@@ -202,12 +210,13 @@ class Build(object):
 
 
 class Commit(object):
-    def __init__(self, sha1, commit_timestamp, describe, log_info, tree):
+    def __init__(self, sha1, commit_timestamp, describe, log_info, gitweb_url, tree):
         self.sha1 = sha1
         self.commit_timestamp = commit_timestamp
         self.describe = describe
         self._builds = []
         self.log_info = log_info
+        self.gitweb_url = gitweb_url
         self.tree = tree
     def add_build(self, config, result, sha1):
         build = Build(config, result, sha1, self)
@@ -237,6 +246,18 @@ class Commit(object):
             prev = self.prev.sha1
         else:
             prev = None
+        commit_url = shortlog_url = None
+        if self.gitweb_url:
+            if 'github' in self.gitweb_url:
+                base_url = self.gitweb_url
+                if base_url.endswith('.git'):
+                    base_url = base_url[:-len('.git')]
+                commit_url = base_url + '/commit/' + self.sha1
+            else:
+                commit_url = self.gitweb_url + ';a=commit;h=' + self.sha1
+                if self.prev:
+                    shortlog_url = '%s;a=shortlog;h=%s;hp=%s' % (
+                        self.gitweb_url, self.sha1, self.prev.sha1)
         return {
             'sha1': self.sha1,
             'describe': self.describe,
@@ -244,6 +265,8 @@ class Commit(object):
             'test_count': sum(len(build['tests']) for build in builds),
             'prev': prev,
             'log_info': self.log_info,
+            'commit_url': commit_url,
+            'shortlog_url': shortlog_url,
             }
 
 
@@ -253,13 +276,13 @@ class Tree(object):
         self._commits = {}
         self.day = day
         self.index = index
-    def get_commit(self, sha1, commit_timestamp, describe, log_info):
+    def get_commit(self, sha1, commit_timestamp, describe, log_info, gitweb_url):
         if sha1 in self._commits:
             # XXX if self._commits[sha1].commit_timestamp != commit_timestamp ...
             return self._commits[sha1]
         else:
             commit_obj = self._commits[sha1] = Commit(
-                sha1, commit_timestamp, describe, log_info, self)
+                sha1, commit_timestamp, describe, log_info, gitweb_url, self)
             return commit_obj
     @property
     def commits(self):
@@ -305,7 +328,7 @@ class Day(object):
             'trees': trees,
             'build_count': sum(tree['build_count'] for tree in trees),
             'test_count': sum(tree['test_count'] for tree in trees),
-            'tree_count': len(trees),
+            'tree_count': len([tree for tree in trees if tree['build_count'] > 0]),
             }
 
 
@@ -314,7 +337,12 @@ class DayCollection(object):
         self.start = start
         self.finish = finish
         self._days = {}
-        d = start
+        # We create a Day object for every day we're interested in, plus one
+        # day older so that we can find the previous build for the last row of
+        # builds displayed.  This isn't quite enough, because the previous
+        # build might have been more than one day before the last one
+        # displayed, but for now it's a reasonable hack.
+        d = start - DAY_DELTA
         while d < finish:
             self._days[d] = Day(d, self)
             d += DAY_DELTA
@@ -335,9 +363,10 @@ class DayCollection(object):
             self._tree2index[git_url] = len(self._tree2index)
         tree = day.get_tree(git_url, self._tree2index[git_url])
         self.all_trees.add(tree)
+        print repr(build_results.gitweb_url)
         commit = tree.get_commit(
             build_results.git_commit_id, build_results.commit_timestamp,
-            build_results.git_describe, build_results.git_log_info)
+            build_results.git_describe, build_results.git_log_info, build_results.gitweb_url)
         config = build_results.config
         if config.endswith('_defconfig'):
             config = config[:-len('_defconfig')]
@@ -395,7 +424,7 @@ class DayCollection(object):
     def evaluate(self):
         connection = DataView.get_connection()
         with contextlib.closing(connection.cursor()) as cursor:
-            cursor.execute(index_sql, (self.start, self.finish))
+            cursor.execute(index_sql, (self.start - DAY_DELTA, self.finish))
 
             for build_result in fetchnamed(cursor):
                 self.add_build(build_result)
@@ -425,6 +454,8 @@ class DayCollection(object):
         day_objs = self._days.values()
         day_objs.sort(key=key)
         day_objs.reverse()
+        # Remove the extra day (see __init__ for the comment about this).
+        del day_objs[-1]
         trees = []
         for url, index in sorted(self._tree2index.items()):
             trees.append({
@@ -449,11 +480,11 @@ def index(request):
     form.is_valid()
     start = form.cleaned_data['start']
     if start is None:
-        start = datetime.date.today() - WEEK_DELTA
+        start = datetime.date.today() - WEEK_DELTA + DAY_DELTA
 
     end = form.cleaned_data['end']
     if end is None:
-        end = datetime.date.today()
+        end = datetime.date.today() + DAY_DELTA
 
     day_collection = DayCollection(start, end)
 
@@ -466,7 +497,7 @@ def index(request):
 
     data = day_collection.json_ready()
     newer_results_link = None
-    if end <= datetime.date.today():
+    if end <= datetime.date.today() + DAY_DELTA:
         newer_start = end
         newer_end = max(
             newer_start + WEEK_DELTA, datetime.date.today())
